
import * as d3 from "d3-color";
import shuffle from "lodash.shuffle";
import random from "random";
import { promises } from "fs";

import * as d3cd from "d3-color-difference";

const PALETTES = new Map([
	[ "random RGB", palette_random ], 
	[ "grayscale", palette_grayscale ],
    [ "monochrome", palette_monochrome ],
    [ "one spot", palette_one_spot ],
    [ "triad", triad_saturated ],
    [ "triad pastel", triad_pastel ],
    [ "triad dusk", triad_dusk ],
  	[ "RGB", palette_rgb ],
  	[ "RBY", palette_rby ],
  	[ "CMY", palette_cmy ],
]);

function palette_random() {
	const u = random.uniform(0, 255);
	return [1,2,3].map((x)=> d3.rgb(u(), u(), u()));
}


function palette_grayscale() {
	const u = random.uniform(0, 1);
	return [1,2,3].map((x)=> d3.hsl(0, 0, u()));
}

function palette_monochrome() {
	const u = random.uniform(0, 1);
	const h = u() * 360;
	return [1,2,3].map((x)=> d3.hsl(h, u(), u()));
}

function palette_one_spot() {
	const hue = random.uniform(0, 360);
	const cols = [ d3.color("white"), d3.color("black"), d3.hsl(hue(), 1, 0.5) ]
	return shuffle(cols);
}

function triad_saturated() {
	return triad(1, 0.5);
}

function triad_pastel() {
	return triad(0.6, 0.7);
}

function triad_dusk() {
	return triad(1, 0.25);
}

function triad(s, l) {
	const u = random.uniform(0, 360);
	const h1 = u();
	const h2 = (h1 + 120) % 360;
	const h3 = (h1 + 240) % 360;
	const cols = [ h1, h2, h3 ].map((h) => d3.hsl(h, s, l));
	return shuffle(cols); 
}

function palette_rgb() {
	const cols = [ d3.rgb(255, 0, 0), d3.rgb(0, 255, 0), d3.rgb(0, 0, 255) ];
	return shuffle(cols);
}

function palette_rby() {
	const cols = [ d3.rgb(255, 0, 0), d3.rgb(255, 255, 0), d3.rgb(0, 0, 255) ];
	return shuffle(cols);
}

function palette_cmy() {
	const cols = [ d3.rgb(0, 255, 255), d3.rgb(255, 255, 0), d3.rgb(255, 0, 255) ];
	return shuffle(cols);
}

class ColourNamer {
	constructor() {
		this.colmap = new Map();
	}

	async load_colours(colourfile) {
		const colourBuff = await promises.readFile(colourfile);
		const colours = colourBuff.toString();
		const seen = new Set(); 
		colours.split(/\n/).map((l) => {
			const m = l.match(/(\d+)\s+(\d+)\s+(\d+)\s+(.*)/);
			if( m ) {
				const sig = `${m[1]},${m[2]},${m[3]}`;
				if( !seen.has(sig) ) {
					this.colmap.set(m[4], d3.rgb(m[1], m[2], m[3]));
					seen.add(sig);
				}
			}
		});
	}

	colour_to_text(d3color) {
		const diffs = []
		for( const [ name, colour] of this.colmap) {
			diffs.push({name: name, dist: d3cd.differenceCie76(colour, d3color)});
		}
		diffs.sort((a, b) => a.dist - b.dist);
		return diffs[0].name;
	}
}


export { PALETTES, ColourNamer }
